@@ -1,4 +1,3 @@
-#!/home/e/anaconda3/bin/python
 """
 rarbccli: RARBG command line interface for scraping the rarbg.to torrent search engine.
 https://github.com/FarisHijazi/rarbgcli
@@ -22,19 +21,31 @@ import os
 import re
 import subprocess
 import sys
+import time
+from requests.utils import quote
+from functools import partial
 from pathlib import Path
-import urllib
+from sys import platform
 
 import requests
 from bs4 import BeautifulSoup
-import time
-from sys import platform
 
 PROGRAM_DIRECTORY = Path(__file__).parent.resolve()
 real_print = print
-print = print if sys.stdout.isatty() else lambda *a, **k: None
+print = print if sys.stdout.isatty() else partial(print, file=sys.stderr)
 COOKIES_PATH = os.path.join(PROGRAM_DIRECTORY, '.cookies.json')
 
+CATEGORY2CODE = {
+    'movies': '48;17;44;45;47;50;51;52;42;46'.split(';'),
+    'xxx': '4'.split(';'),
+    'music': '23;24;25;26'.split(';'),
+    'tvshows': '18;41;49'.split(';'),
+    'software': '33;34;43'.split(';'),
+    'games': '27;28;29;30;31;32;40;53'.split(';'),
+    'nonxxx': '2;14;15;16;17;21;22;42;18;19;41;27;28;29;30;31;32;40;23;24;25;26;33;34;43;44;45;46;47;48;49;50;51;52;54'.split(';'),
+    '': '',
+}
+
 
 ## Captcha solving taken from https://github.com/confident-hate/seedr-cli
 
@@ -66,8 +77,8 @@ def solveCaptcha(threat_defence_url):
             png = driver.get_screenshot_as_png()
             x = location['x']
             y = location['y']
-            width = location['x']+size['width']
-            height = location['y']+size['height']
+            width = location['x'] + size['width']
+            height = location['y'] + size['height']
             im = Image.open(BytesIO(png))
             im = im.crop((int(x), int(y), int(width), int(height)))
             return pytesseract.image_to_string(im)
@@ -76,15 +87,17 @@ def solveCaptcha(threat_defence_url):
     options.add_argument('--no-sandbox')
     options.add_argument("--headless")
     options.add_argument("--log-level=3")
+    options.add_argument("--disable-logging");
+    options.add_argument("--output=" + ('NUL' if sys.platform == 'win32' else '/dev/null'));
 
     driver = webdriver.Chrome(
         chrome_options=options,
         # chrome_profile=FFprofile,
-        # service_log_path=StringIO
-        )
+        service_log_path=('NUL' if sys.platform == 'win32' else '/dev/null')
+    )
     driver.implicitly_wait(10)
     driver.get(threat_defence_url)
-    
+
     if platform == 'win32':
         pytesseract.pytesseract.tesseract_cmd = os.path.join(PROGRAM_DIRECTORY, 'Tesseract-OCR', 'tesseract')
 
@@ -104,8 +117,7 @@ def solveCaptcha(threat_defence_url):
         print(e)
 
     time.sleep(3)
-    selCookie = driver.get_cookies()
-    cookies = {c['name']: c['value'] for c in selCookie}
+    cookies = {c['name']: c['value'] for c in (driver.get_cookies())}
     driver.close()
     return cookies
 
@@ -145,12 +157,15 @@ def deal_with_threat_defence(threat_defence_url):
 
 
 def get_page_html(target_url, cookies):
+    headers = {
+        'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36'
+    }
     while True:
         r = requests.get(target_url, headers=headers, cookies=cookies)
-        print('going to page', r.url)
+        print('going to page', r.url, end=' ')
         if 'threat_defence.php' not in r.url:
             break
-        print('defence detected')
+        print('\ndefence detected')
         cookies = deal_with_threat_defence(r.url)
         # save cookies to json file
         with open(COOKIES_PATH, 'w') as f:
@@ -170,7 +185,7 @@ def extract_magnet(anchor):
     trackers = 'http%3A%2F%2Ftracker.trackerfix.com%3A80%2Fannounce&tr=udp%3A%2F%2F9.rarbg.me%3A2710&tr=udp%3A%2F%2F9.rarbg.to%3A2710';
     try:
         hash = re.search(regex, str(anchor))[1]
-        title = urllib.parse.quote(anchor.get('title'))
+        title = quote(anchor.get('title'))
         return f'magnet:?xt=urn:btih:{hash}&dn={title}&tr={trackers}'
     except Exception as e:
         return ''
@@ -183,12 +198,12 @@ def parse_size(size):
 
 
 def dict_to_fname(d):
-    # copy and santitize
+    # copy and sanitize
     args_dict = {k: str(v).replace('"', '').replace(',', '') for k, v in vars(d).items()}
     del args_dict['sort']
     del args_dict['magnet']
     del args_dict['domain']
-    del args_dict['no_cache']
+    del args_dict['cache']
     filename = json.dumps(args_dict, indent=None, separators=(',', '='), ensure_ascii=False)[1:-1].replace('"', '')
     return filename
 
@@ -201,82 +216,128 @@ def open_program(program):
         return subprocess.call([opener, program])
 
 
-cat_code_dict = {
-    'movies': '48;17;44;45;47;50;51;52;42;46'.split(';'),
-    'xxx': '4'.split(';'),
-    'music': '23;24;25;26'.split(';'),
-    'tvshows': '18;41;49'.split(';'),
-    'software': '33;34;43'.split(';'),
-    'games': '27;28;29;30;31;32;40;53'.split(';'),
-    'nonxxx': '2;14;15;16;17;21;22;42;18;19;41;27;28;29;30;31;32;40;23;24;25;26;33;34;43;44;45;46;47;48;49;50;51;52;54'.split(';'),
-    '': '',
-}
+def get_user_input_interactive(torrent_dicts):
+    header = ' '.join(["SN".ljust(4), "TORRENT NAME".ljust(80), "SEEDS".ljust(6), "LEECHES".ljust(6), "SIZE".center(12), "UPLOADER"])
+    choices = []
+    for i in range(len(torrent_dicts)):
+        torrent_name = str(torrent_dicts[i]['title'])
+        torrent_size = str(torrent_dicts[i]['size'])
+        torrent_seeds = str(torrent_dicts[i]['seeders'])
+        torrent_leeches = str(torrent_dicts[i]['leechers'])
+        torrent_uploader = str(torrent_dicts[i]['uploader'])
+        choices.append({
+            'value': int(i),
+            'name': ' '.join([str(i + 1).ljust(4), torrent_name.ljust(80), torrent_seeds.ljust(6), torrent_leeches.ljust(6), torrent_size.center(12),
+                              torrent_uploader])
+        })
 
-headers = {
-    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
-    "cache-control": "max-age=0",
-    "sec-ch-ua-mobile": "?0",
-    "sec-fetch-dest": "document",
-    "sec-fetch-mode": "navigate",
-    "sec-fetch-site": "same-origin",
-    "sec-fetch-user": "?1",
-    "upgrade-insecure-requests": "1"
-}
+    from prompt_toolkit import styles
+    import questionary
+    prompt_style = styles.Style([
+        ('qmark', 'fg:#5F819D bold'),
+        ('question', 'fg:#289c64 bold'),
+        ('answer', 'fg:#48b5b5 bold'),
+        ('pointer', 'fg:#48b5b5 bold'),
+        ('highlighted', 'fg:#07d1e8'),
+        ('selected', 'fg:#48b5b5 bold'),
+        ('separator', 'fg:#6C6C6C'),
+        ('instruction', 'fg:#77a371'),
+        ('text', ''),
+        ('disabled', 'fg:#858585 italic')
+    ])
+    answer = questionary.select(
+        header + '\nSelect torrents',
+        choices=choices,
+        style=prompt_style
+    ).ask()
+    return [answer]
 
-target_url = 'https://{domain}/torrents.php?search={search}&order={order}&category={category}&page={page}&by={by}'
 
-def main():
+def get_args():
     orderkeys = ["data", "filename", "leechers", "seeders", "size", ""]
     sortkeys = ["title", "date", "size", "seeders", "leechers", ""]
-
     parser = argparse.ArgumentParser(__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-    parser.add_argument('search', help='search term')
-    parser.add_argument('--category', '-c', choices=cat_code_dict.keys(), default='')
-    parser.add_argument('--limit', '-l', type=float, default='inf', help='limit number of torrent magnet links')
-    parser.add_argument('--domain', '-d', default='rarbgunblocked.org', help='domain to search, you could put an alternative mirror domain here')
-    parser.add_argument('--order', '-r', choices=orderkeys, default='', help='order results (before query) by this key. empty string means no sort')
-    parser.add_argument('--descending', action='store_true', help='order in descending order (only available for --order)')
-    parser.add_argument('--magnet', '-m', action='store_true', help='output magnet links')
-    parser.add_argument('--sort', '-s', choices=sortkeys, default='', help='sort results (after scraping) by this key. empty string means no sort')
-    parser.add_argument('--no_cache', '-nc', action='store_true', help='don\'t use cached results from previous searches')
+    parser.add_argument('search', help='Search term')
+    parser.add_argument('--category', '-c', choices=CATEGORY2CODE.keys(), default='')
+    parser.add_argument('--limit', '-l', type=float, default='inf', help='Limit number of torrent magnet links')
+    parser.add_argument('--domain', '-d', default='rarbgunblocked.org', help='Domain to search, you could put an alternative mirror domain here')
+    parser.add_argument('--order', '-r', choices=orderkeys, default='', help='Order results (before query) by this key. empty string means no sort')
+    parser.add_argument('--descending', action='store_true', help='Order in descending order (only available for --order)')
+    parser.add_argument('--interactive', '-i', action='store_true', help='Interactive mode, show menu of torrents')
+
+    parser.add_argument('--magnet', '-m', action='store_true', help='Output magnet links')
+    parser.add_argument('--sort', '-s', choices=sortkeys, default='', help='Sort results (after scraping) by this key. empty string means no sort')
+
+    parser.add_argument('--cache', action='store_true', help='Use cached results from previous searches')
+    parser.add_argument('--no_cookie', '-nk', action='store_true',
+                        help='Don\'t use CAPTCHA cookie from previous runs (will need to resolve a new CAPTCHA)')
     args = parser.parse_args()
-    assert args.limit >= 1, '--limit must be greater than 1'
-    if args.descending:
-        assert args.order, '--descending requires --order'
+    if args.interactive and not sys.stdout.isatty():
+        print('--interactive mode requires a TTY (cannot be piped)', file=sys.stderr)
+        exit(1)
+    if not args.limit >= 1:
+        print('--limit must be greater than 1', file=sys.stderr)
+        exit(1)
+    if args.descending and not args.order:
+        print('--descending requires --order', file=sys.stderr)
+        exit(1)
+    return args
+
+
+def main():
+    def print_results(dicts):
+        if args.sort:
+            dicts.sort(key=lambda x: x[args.sort], reverse=True)
+        if args.limit < float('inf'):
+            dicts = dicts[:int(args.limit)]
+
+        for d in dicts:
+            if not d['magnet']:
+                print('fetching magnet link for', d['title'])
+                html_subpage = requests.get(d['href'], cookies=cookies).text.encode('utf-8')
+                parsed_html_subpage = BeautifulSoup(html_subpage, 'html.parser')
+                d['magnet'] = parsed_html_subpage.select_one('a[href^="magnet:"]').get('href')
+                d['torrent_file'] = parsed_html_subpage.select_one('a[href^="/download.php"]').get('href')
+
+        if args.magnet:
+            real_print('\n'.join([t['magnet'] for t in dicts]))
+        else:
+            real_print(json.dumps(dicts, indent=4))
+
+    args = get_args()
 
     out_history_fname = dict_to_fname(args)
     os.makedirs(os.path.join(PROGRAM_DIRECTORY, '.history'), exist_ok=True)
-
     out_history_path = os.path.join(PROGRAM_DIRECTORY, '.history', out_history_fname + '.json')
-    if not args.no_cache:
-        if os.path.exists(out_history_path):
-            with open(out_history_path, 'r') as f:
-                history = json.load(f)
-            print('using cached results from', out_history_path)
-            dicts_all = history
-            if args.magnet:
-                real_print('\n'.join([t['magnet'] for t in dicts_all]))
-            else:
-                real_print(json.dumps(dicts_all))
-            sys.exit(0)
 
-    # read cookies from json file
+    if args.cache and os.path.exists(out_history_path):
+        with open(out_history_path, 'r') as f:
+            history = json.load(f)
+        print('Using cached results from', out_history_path)
+        print_results(history)
+        sys.exit(0)
+
+    # make empty cookie if cookie doesn't already exist
     if not os.path.exists(COOKIES_PATH):
         with open(COOKIES_PATH, 'w') as f:
             json.dump({}, f)
-    with open(COOKIES_PATH, 'r') as f:
-        cookies = json.load(f)
+
+    # read cookies from json file
+    cookies = {}
+    if not args.no_cookie:
+        with open(COOKIES_PATH, 'r') as f:
+            cookies = json.load(f)
 
     magnets = []
     torrents_all = []
     dicts_all = []
     i = 1
-    while True:
-        print('scraping page', i)
-        r, html, cookies = get_page_html(
-            target_url.format(domain=args.domain, search=args.search, order=args.order, category=cat_code_dict[args.category], page=i,
-                              by='DESC' if args.descending else 'ASC'),
-                              cookies=cookies)
+    while True:  # for all pages
+        target_url = 'https://{domain}/torrents.php?search={search}&order={order}&category={category}&page={page}&by={by}'
+        r, html, cookies = get_page_html(target_url.format(
+            domain=args.domain, search=args.search, order=args.order, category=CATEGORY2CODE[args.category], page=i,
+            by='DESC' if args.descending else 'ASC'
+        ), cookies=cookies)
 
         with open(os.path.join(PROGRAM_DIRECTORY, '.history', out_history_fname + f'_torrents_{i}.html'), 'w', encoding='utf8') as f:
             f.write(r.text)
@@ -293,39 +354,57 @@ def main():
             break
         magnets += list(map(extract_magnet, torrents))
         # removed torrents and magnet links that have empty magnets, but maintained order
-        torrents, magnets = zip(*[[a, m] for (a, m) in zip(torrents, magnets) if m])
+        torrents, magnets = zip(*[[a, m] for (a, m) in zip(torrents, magnets)])
         torrents, magnets = list(torrents), list(magnets)
 
-        dicts_all += [{
-            'title': torrent.get('title'),
-            'href': torrent.get('href'),
+        dicts_current = [{
+            'title': torrent.get("title"),
+            'href': f"https://{args.domain}{torrent.get('href')}",
             'date': datetime.datetime.strptime(str(torrent.findParent('tr').select_one('td:nth-child(3)').contents[0]),
                                                '%Y-%m-%d %H:%M:%S').timestamp(),
             'size': parse_size(torrent.findParent('tr').select_one('td:nth-child(4)').contents[0]),
             'seeders': int(torrent.findParent('tr').select_one('td:nth-child(5) > font').contents[0]),
             'leechers': int(torrent.findParent('tr').select_one('td:nth-child(6)').contents[0]),
+            'uploader': str(torrent.findParent('tr').select_one('td:nth-child(8)').contents[0]),
             'magnet': magnet,
         } for (torrent, magnet) in zip(torrents, magnets)]
 
+        dicts_all += dicts_current
         torrents_all += torrents
+
+        if args.interactive:
+            while True:
+                os.system('cls||clear')
+                user_input = get_user_input_interactive(dicts_current)
+                if not user_input:  # next page
+                    print("\nNo item selected\n")
+                    pass
+                else:  # indexes
+                    input_indexes = [int(x) for x in user_input]
+                    # save history to json file
+                    with open(out_history_path, 'w', encoding='utf8') as f:
+                        json.dump(dicts_all, f, indent=4)
+                    print_results([dicts_current[idx] for idx in input_indexes])
+
+                user_input = input("[ENTER]: continue to the next page, [b]: go (b)ack to results, [q]: to (q)uit: ")
+                if user_input.lower() == 'b':
+                    continue
+                elif user_input.lower() == 'q':
+                    exit(0)
+                elif user_input == '':
+                    break
+
+        # save history to json file
+        with open(out_history_path, 'w', encoding='utf8') as f:
+            json.dump(dicts_all, f, indent=4)
+
         if len(list(filter(None, magnets))) >= args.limit:
             print(f'reached limit {args.limit}, stopping')
             break
         i += 1
 
-    if args.sort:
-        dicts_all.sort(key=lambda x: x[args.sort], reverse=True)
-    if args.limit < float('inf'):
-        dicts_all = dicts_all[:int(args.limit)]
-
-    if args.magnet:
-        real_print('\n'.join([t['magnet'] for t in dicts_all]))
-    else:
-        real_print(json.dumps(dicts_all))
-
-    # save history to json file
-    with open(out_history_path, 'w', encoding='utf8') as f:
-        json.dump(dicts_all, f, indent=4)
+    if not args.interactive:
+        print_results(dicts_all)
 
 
 if __name__ == '__main__':
